## -*- coding: utf-8 -*-
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
$Id: FSSTestCase.py,v 1.2 2005/03/30 14:21:12 clebeaupin Exp $
"""

# Python imports
import os
import time
import Globals

# Zope imports
from Testing import ZopeTestCase
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import noSecurityManager

# CMF imports
from Products.CMFCore.utils import getToolByName

# Plone imports
from Products.PloneTestCase import PloneTestCase

# Products imports
from Products.FileSystemStorage.config import INSTALL_EXAMPLE_TYPES_ENVIRONMENT_VARIABLE

# Globals
portal_name = 'portal'
portal_owner = 'portal_owner'
default_user = PloneTestCase.default_user
default_password = PloneTestCase.default_password

STORAGE_PATH = os.path.join(Globals.INSTANCE_HOME, 'var', 'fsstmp')

DATA_PATH = os.path.join(Globals.INSTANCE_HOME, 'Products', 'FileSystemStorage', 'tests', 'data')
CONTENT_PATH = os.path.join(DATA_PATH, 'word.doc')
IMAGE_PATH = os.path.join(DATA_PATH, 'image.jpg')
CONTENT_TXT = """mytestfile"""

class FSSTestCase(PloneTestCase.PloneTestCase):

    class Session(dict):
        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()
        base_path = STORAGE_PATH
        
        # Create temporary dir to run test cases
        if not os.path.exists(base_path):
            os.mkdir(base_path)
            
        # Configure tool
        self.fss_tool = getToolByName(self.portal, 'portal_fss')
        self.fss_tool.storage_path = STORAGE_PATH
        self.fss_tool.backup_path = STORAGE_PATH
        
        # Patch getStorageStragegy to test all strategies
        strategy_klass = self.strategy_klass
        def getStorageStrategy(self):
            return strategy_klass(self.getStoragePath(), self.getBackupPath())
        
        from Products.FileSystemStorage.FSSTool import FSSTool
        FSSTool.getStorageStrategy = getStorageStrategy
        
        # Check if fss is switched
        self.use_atct = False
        ttool = getToolByName(self.portal, 'portal_types')
        info = ttool.getTypeInfo('Folder')
        if info.getProperty('meta_type') == 'ATFolder':
            self.use_atct = True
        
    def beforeTearDown(self):
        """Remove all the stuff again.
        """
        base_path = STORAGE_PATH
        for root, dirs, files in os.walk(base_path, topdown=None):
            for name in files:
                os.remove(os.path.join(root, name))
            for name in dirs:
                os.rmdir(os.path.join(root, name))
        if os.path.exists(base_path):
            os.rmdir(base_path)

    def getDataPath(self):
        """Returns data path used for test cases"""
    
        return DATA_PATH
    
    def loginAsPortalOwner(self):
        '''Use if you need to manipulate an article as member.'''
        uf = self.app.acl_users
        user = uf.getUserById(portal_owner).__of__(uf)
        newSecurityManager(None, user)

    def addFileByString(self, folder, content_id):
        """Adds a file by string.
        """
        
        folder.invokeFactory('FSSItem', id=content_id)
        content = getattr(folder, content_id)
        get_transaction().commit(1)
        kw = {'file' : CONTENT_TXT}
        content.edit(**kw)
        return content

    def addFileByFileUpload(self, folder, content_id):
        """Adds a file by file upload.
        """
        
        folder.invokeFactory('FSSItem', id=content_id)
        content = getattr(folder, content_id)
        get_transaction().commit(1)
        self.updateContent(content, 'file', CONTENT_PATH)
        return content
        
    def addImageByFileUpload(self, folder, content_id):
        """
        Adding image
        """
        folder.invokeFactory('FSSItem', id=content_id)
        content = getattr(folder, content_id)
        get_transaction().commit(1)
        self.updateContent(content, 'image', IMAGE_PATH)
        return content

    def updateContent(self, content, field, filepath):
        """Updates a field content for a file.
        """
        
        from dummy import FileUpload
        file = open(filepath, 'rb')
        file.seek(0)
        filename = filepath.split('/')[-1]
        fu = FileUpload(filename=filename, file=file)
        kw = {field: fu}
        content.edit(**kw)

# Install FSS Example types
os.environ[INSTALL_EXAMPLE_TYPES_ENVIRONMENT_VARIABLE] = 'True' 
ZopeTestCase.installProduct('PortalTransforms')
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('FileSystemStorage')

HAS_ATCT = True
try:
    ZopeTestCase.installProduct('ATContentTypes')
except:
    HAS_ATCT = False

# Setup Plone site
if HAS_ATCT:
    PloneTestCase.setupPloneSite(products=['Archetypes', 'FileSystemStorage'])
else:
    PloneTestCase.setupPloneSite(products=['Archetypes', 'FileSystemStorage', 'ATContentTypes'])
